#!/usr/bin/python

import sys
import os
import math
import string

def H(v):
    vals = {}
    for i in v:
        if type(i) == type([]):
            i = tuple(i)
        vals[i] = vals.get(i, 0.0) + 1.0

    ret = 0.0
    for val in vals:
        p = vals[val] / len(v)
        ret -= p * log2(p)

    return ret

def H2(data):
    vals = []
    for row in data:
        vals += row

    ret = 0.0
    for val in vals:
        p = val * 1.0 / sum(vals)
        ret -= p * log2(p)
    return ret

def log2(a):
    try:
        return math.log(a) / math.log(2)
    except:
        return 0

def getData(filename):
    ret = []
    for line in open(filename):
        ret.append(map(string.strip, line.split(",")))
    return ret

def calcMI(data, i, j):
    counts = {}

    ivals = {}
    jvals = {}
    for row in data:
        if row[i] in "abcdef" and row[j] in "abcdef":
            ci = counts.get(row[i], {})
            ci[row[j]] = ci.get(row[j], 0) + 1
            counts[row[i]] = ci

            ivals[row[i]] = 1
            jvals[row[j]] = 1

    if not counts: return 0.0
    
    ikeys = ivals.keys()
    jkeys = jvals.keys()

    ikeys.sort()
    jkeys.sort()

    for ik in ikeys:
        if ik not in counts:
            counts[ik] = {}

        for jk in jkeys:
            if jk not in counts[ik]:
                counts[ik][jk] = 0

    table = [[counts[x][y] for x in ikeys] for y in jkeys]    
    xs = [row[i] for row in data]
    ys = [row[j] for row in data]

    return round(H(xs) + H(ys) - H2(table), 3)

def main():
    data = getData(sys.argv[1])

    try:
        outfile = open(sys.argv[2], 'w')
    except:
        outfile = sys.stdout

    results = []
    for i in range(1, len(data[0])):
        cur = []
        for j in range(1, len(data[0])):
            cur.append(calcMI(data, i, j))
        results.append(cur)

    for row in results:
        row = map(str, row)
        print >>outfile, string.join(row, ",")

if __name__ == "__main__":
    main()
    
    
